#ifndef _VIDEO_C_
#define _VIDEO_C_

#include <gst/gst.h>
#include <glib.h>
#include <hildon/hildon-program.h>
#include <gst/interfaces/xoverlay.h>
#include <gdk/gdk.h>

#include "functionality.h"
#include "log.h"

#define INTROVIDEODIR "/usr/share/mobikid/video/intro/"

/* ==== Helper functions ==== */
void video_initialize();
void video_play();
void video_deinitialize();
void video_stop();
void create_video_pipeline( gchar *name );
gboolean create_window( GstBus *bus, GstMessage *message, gpointer data );

GstElement *video_pipeline = NULL;
GtkWidget *video_screen = NULL;
gchar *video_filename = NULL;
GstElement *video_source = NULL;

/**
 @return true is a file was found in the directory video/intro
		 otherwise false
  the variable video_filename contains the name of first file (alphabetical order) of the directory video/intro
**/
gboolean video_determine_file()
{
	#if DEBUG	
	g_print( "determine video\n" );	
	#endif

	GDir *video_dir = NULL;
	GError *error = NULL;
	guint flags = 0;
	
	video_dir = g_dir_open( (char *) INTROVIDEODIR, flags, &error );
	 
	if( video_dir == NULL ) 
	{
		g_warning("Couldn't not open the video directory\n");
		g_warning("(glib) Error: %s\n", error->message );

		g_error_free( error );

		return FALSE;
	}

    const gchar *name = NULL;
	/* the return value of g_dir_read_name is own by glib
			Do not modify or free 
	*/

	//detect filename
	name = (gchar *) g_dir_read_name( video_dir );  
	
	if( name == NULL ) {
		g_warning("Couldn't find a video in the directory\n");
		g_free( name );

		return FALSE;
	}
		
	/* String is used as helper to add URI for the video */
	GString *dir_string = NULL;

	dir_string = g_string_new( INTROVIDEODIR );	
	g_string_prepend( dir_string, "file:///" );
	
	dir_string = g_string_append( dir_string, name );
	 
	video_filename =  g_string_free( dir_string, FALSE );

	#if DEBUG	
		g_print("filename: %s\n", video_filename);
	#endif

	g_dir_close( video_dir );	
	return TRUE;
}

void video_playback_start( GtkWidget *screen )
{
	g_message( "Play video %s", video_filename);
	video_screen = screen;
	video_initialize();
	video_play();
}

void video_playback_stop()
{
	#if DEBUG
		g_print("stop video playback\n");
	#endif
	if( video_pipeline != NULL){
		video_stop();
		video_deinitialize();
	}		
}

/**
	destroys the screen in which the video got displayed
	is needed because if the screen is not destroyed it might be displayed on the splash screen as white background
**/
void video_cleanup()
{
	#if DEBUG
		g_print("Video clean up:\n");	
	#endif
	gtk_widget_destroy( video_screen );	
	video_screen = NULL;
	video_source = NULL;	
	g_free( video_filename );
	video_filename = NULL;		
}
/**************************** helper functions *********************************/

/************************** begin: start functions *************************/
void video_initialize()
{
	g_assert( video_pipeline == NULL );

	/* =====================
		initialize video
	*/
	#if DEBUG
		g_print("Video initialize:\n");	
	#endif

	create_video_pipeline( "video-pipeline" );
	
    g_object_set( G_OBJECT(video_pipeline), "uri", video_filename, NULL );
	
	gst_element_set_state( video_pipeline, GST_STATE_PAUSED );	
	
	if( video_pipeline == NULL ) {
		g_warning("Something failed while creating a pipeline\n");
	}
}

void video_play() 
{
	#if DEBUG
		g_print("play video\n");	
	#endif

	g_assert( video_pipeline != NULL );

	gst_element_set_state( video_pipeline, GST_STATE_PLAYING );
}

/**************************** end: start functions *************************/

/************************** begin: stop functions *************************/

void video_deinitialize()
{
	#if DEBUG
		g_print( "video_deinitialize()\n" );
	#endif

	gst_element_set_state( video_pipeline, GST_STATE_NULL );

	g_object_unref( video_pipeline );
	video_pipeline = NULL;

	//
}

void video_stop() 
{
	g_assert( video_pipeline != NULL );

	
}
/**************************** end: stop functions *************************/

//pipeline functions

void create_video_pipeline( gchar *name )
{
	/* =====================
		Create video pipeline
	*/
	#if DEBUG
		g_print("Create pipeline:\n");
	#endif

	GstBus *bus = NULL;

	video_pipeline = gst_element_factory_make( "playbin", name );
	if( video_pipeline == NULL ) {
		g_warning("[%s] Couldn't create pipeline\n", name );
	}

	bus = gst_pipeline_get_bus( GST_PIPELINE(video_pipeline) );
	if( bus == NULL ) {
		g_warning("[%s] Couldn't get bus from pipeline\n", name );
	}
		
    gst_bus_add_watch( bus, create_window, video_pipeline);	
	g_object_unref( bus );

	gst_element_set_state( video_pipeline, GST_STATE_NULL );
	
	g_object_set( G_OBJECT(video_pipeline), "volume", 8.0, NULL );
}


gboolean create_window (GstBus *bus, GstMessage *message, gpointer data)
{
  	
 	if( GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS){
		#if DEBUG
			g_print( "End of video playback\n");
		#endif
	
		from_splash_screen_to_selection() ;
		//Slide to next window
		return TRUE;		
	}
 	// ignore anything but 'prepare-xwindow-id' element messages
 	if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
   		return TRUE;
 
 	if (!gst_structure_has_name (message->structure, "prepare-xwindow-id"))
   		return TRUE;

 	gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (GST_MESSAGE_SRC (message)), GDK_DRAWABLE_XID(video_screen->window));
 	 	
	return TRUE;
}
 
#endif
